<?php

# revision : $Rev: 73 $
# auteur   : $Author: marc-travail $
# Date     : $Date: 2013-02-11 22:11:09 +0100 (lun., 11 févr. 2013) $
# URL      : $HeadURL: svn+ssh://subversion@svn.vous-et-nous.com/home/subversion/repository/formations/utils_presentation/class/parse_xml.class.php $

class parse_xml extends html_entities {

  private $document_extension = '';
  private $base_url;
  private $url_list = array();
  private $error_string = '';
  private $html_body = '';
  private $html_header = '';
  static private $starthtml = <<<THE_HEADER
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr">
THE_HEADER;
  static private $endhtml = <<<THE_TRAILER
    <div class="validator">pages validées par
       <a href="http://validator.w3.org/check?uri=referer"><img src="images/valid_xhtml.png" alt="Valid XHTML 1.1" title="xHTML 1.1 validé !" /></a>
       <a href="http://jigsaw.w3.org/css-validator/check/referer"><img src="images/valid_css.png" alt="CSS validé !" title="CSS validé !"/></a>
    </div>
  </body>
</html>
THE_TRAILER;

 /**
   * Cette procédure vérifie si la chaîne passée en argument ne contient pas d'erreur XML
   * Si aucune erreur le retour sera true
   * Si des erreurs sont présentes le retour sera la chaîne de caractères indiquant les erreurs.
   */
  function __construct($url, $type = '') {
    $this->base_url = $url;
    $this->document_extension = $type;
    $this->get_url_list($url);
    $this->get_url_body($url);
    $this->get_url_end();
  }

 /**
   * Cette procédure vérifie si la chaîne passée en argument ne conient pas d'erreur XML
   * Si aucune erreur le retour sera true
   * Si des erreurs sont présentes le retour sera la chaîne de caractères indiquant les erreurs.
   */
  private static function check_xml_string($data, $url) {
    $res = true;
    $xml_parser = xml_parser_create();
    if (!xml_parse($xml_parser, $data, true)) {
      $res = sprintf("<p>erreur XML(<samp>%s</samp>) : %s at line %d</p>\n", $url,
		     xml_error_string(xml_get_error_code($xml_parser)),
		     xml_get_current_line_number($xml_parser));
    }
    xml_parser_free($xml_parser);
    return $res;
  }

  /**
   * Cette procédure retourne un objet parsable en XML sauf erreur d'analyse
   * En cas d'erreur la variable $this->error_string sera augmenté du messagrge d'erreur et la valeur de retour sera false
   * En cas de succes la valeur de retour sera l'objet simplexml à analyser
   */
  public function load_xml_url($url) {
    $file = file($url);
    $xmlstrg = join('', self::html_entities_to_utf8($file));
    if (($res = self::check_xml_string($xmlstrg, $url)) !== true) {
      $this->error_string .= $res;
    } elseif (($res = simplexml_load_string($xmlstrg)) === false) {
      $this->error_string .= sprintf("<h2>Erreur sur URL : <q>%s</q></h2>\n", $url);
    } else {
      return $res;
    }
    return false;
  }
  
  /**
   * Cette procédure augmente la chaine $this->html_body des chapitres trouvés dans l'objet simplexml passé en argument.
   */
  public function get_html_chapters($xml) {
    $xml->registerXPathNamespace('xhtml', 'http://www.w3.org/1999/xhtml');
    $hnode = $xml->xpath("/xhtml:html/xhtml:body/xhtml:div[@id='slide0']");
    if ($hnode) {
      $hnode[0]['class'] = 'slidechapter';
      unset($hnode[0]['id']);
      $this->html_body .= $hnode[0]->asXML();
      $this->html_body .= "\n\n    ";
    }
      
    foreach($xml->xpath("/xhtml:html/xhtml:body/xhtml:div[contains(concat(' ',normalize-space(@class),' '),' slide ') and not(@id='slide0')]") as $node) {
      $this->html_body .= $node->asXML();
      $this->html_body .= "\n\n    ";
    }
  }
  
  /**
   * Cette procédure initialise $this->$url_list avec la liste des URL trouvées dans le document passé en argument.
   */
  public function get_url_list($url) {
    if (($s = $this->load_xml_url($url)) !== false) {
      $s->registerXPathNamespace('xhtml', 'http://www.w3.org/1999/xhtml');
      $links = $s->xpath("/xhtml:html/xhtml:body//xhtml:a");       // récupération des liens
      if ($links) {
	$nchapter=1;
	foreach ($links as $link) {
	  $lnk = (string)$link['href'];
	  if ($this->document_extension == '' || preg_match('/'.$this->document_extension.'$/', $lnk)) {
	    //echo $lnk."/-----<br/>\n";
	    $this->url_list[] = $lnk;
	    $link['href'] = '#chapter_'.$nchapter++; // subsitution des liens externes vers ancres internes
	  }
	}
      }
      $this->html_header = self::$starthtml;
      $script = $s->xpath("/xhtml:html/xhtml:head/xhtml:script");
      unset($script[0][0]);                                        // élimination de la balise script
      // insertion de prepre de la chaîne suivante
      // <link type="image/x-icon" rel="shortcut icon" href="images/icon.png"/>
      $head = $s->xpath("/xhtml:html/xhtml:head");                 // récupération de l'entête
      $icon = $head[0]->addChild('link');
      $icon->addAttribute('type', 'image/x-icon');
      $icon->addAttribute('rel', 'shortcut icon');
      $icon->addAttribute('href', 'images/icon_print.png');
      $this->html_header .= $head[0]->asXML();
      $title = $s->xpath("/xhtml:html/xhtml:body/xhtml:div[@id='slide0']/xhtml:h2");
      $subtitle = $s->xpath("/xhtml:html/xhtml:body/xhtml:div[@id='slide0']/xhtml:h1");
      $this->html_header .= "\n";
      $this->html_body .= "  <body>\n";

      $this->html_body .= '    <div id="slide0"><h1>';
      $this->html_body .= dom_import_simplexml($title[0])->textContent;
      $this->html_body .= "</h1></div>\n\n    ";

      $chapt_num = 1;                                              // affichage de la table des matières
      $this->html_body .= '<div id="summary"><h2>';
      $this->html_body .= dom_import_simplexml($subtitle[0])->textContent;
      $this->html_body .= "</h2>\n";
      foreach($s->xpath("/xhtml:html/xhtml:body/xhtml:div/xhtml:div/xhtml:ul/xhtml:li") as $item) {
	$nitem = simplexml_load_string($item->asXML()); 
	$nchapts = count($nitem->xpath("//a"));
	if ($nchapts) {
	  $num_text = array();
	  for ($i = 0; $i < $nchapts; $i++) $num_text[] = $chapt_num++;
	  $pluriel = ($nchapts > 1) ? 's' : '';
	  $this->html_body .= sprintf("    <h3>Chapitre%s <em>%s</em></h3>\n", $pluriel, implode(', ', $num_text));
	  $this->html_body .= "    <ul>\n      ";
	  $this->html_body .= $item->asXML();
	  $this->html_body .= "\n    </ul>\n";
	}
      }
      $this->html_body .= "</div>\n\n    ";
      return true;
    } else {
      return false;
    }
  }

  /**
   * Cette procédure crée la chaîne de caractères à partir de $this->$url_list.
   */
  public function get_url_body() {
    $xml_array = array();
    foreach($this->url_list as $url) {
      if (($s = $this->load_xml_url($url)) !== false) {
	$xml_array[] = $s;
      }
    }
    
    if ($this->error_string === '') {
      $nchapter = 1;
      foreach($xml_array as $s) {
	$s->registerXPathNamespace('xhtml', 'http://www.w3.org/1999/xhtml');
	$hnode = $s->xpath("/xhtml:html/xhtml:body/xhtml:div[@id='slide0']");
	if ($hnode) {
	  $hnode[0]['class'] = 'slidechapter';
	  $hnode[0]['id'] = 'chapter_'.$nchapter++;
	  //unset($hnode[0]['id']);
	  $this->html_body .= $hnode[0]->asXML();
	  $this->html_body .= "\n\n    ";
	}
    
	foreach($s->xpath("/xhtml:html/xhtml:body/xhtml:div[contains(concat(' ',normalize-space(@class),' '),' slide ') and not(@id='slide0')]") as $node) {
	  $this->html_body .= $node->asXML();
	  $this->html_body .= "\n\n    ";
	}
      }
      return true;
    } else {
      return false;
    }
  }

  /**
   * Cette procédure crée la fin de fichier html.
   */
  public function get_url_end() {

    $selfurl = 'http://'.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'];
    $this->html_body .=
      '<div class="no_print help_print"><h2>Aide à la génération de documents</h2>'."\n";
    $this->html_body .=
      sprintf("<h3>Pour mettre à plat l'arborescence du site (Sous UNIX) :</h3>\n".
	      "<pre class=\"full\"><code>wget -r -nH --cut-dirs=3 --restrict-file-names=windows -E -k \\\n'%s'</code></pre>\n",
	      $this->base_url);
    $this->html_body .=
      sprintf("<h3>Pour ne mettre à plat que les exercices (Sous UNIX) :</h3>\n".
	      "<pre class=\"full\"><code>wget -r -nH --cut-dirs=3 --restrict-file-names=windows -E -k \\\n'%s'</code></pre>\n",
	      dirname($this->base_url).'/exercices/?solution&amp;nocrypt');
    $this->html_body .= "<h3>Pour compresser les exercices au format <em>zip</em> :</h3>\n<pre class=\"full\"><code>zip -r exercices.zip exercices</code></pre>\n";
    $this->html_body .= sprintf("<h3>Pour générer un document d'impression</h3>\n<ul>");
    foreach(glob('./css/print_*.css') as $path) { 
      $orgaform = preg_replace('/^.*print_([a-zA-Z0-9_]+)\.css$/', '$1', $path);
      $rtn = " \\\n       ";
      $this->html_body .= sprintf("<li>Pour <a href=\"%s?pdf=%s\">%s</a> :", $selfurl, $orgaform, $orgaform);
      $this->html_body .= sprintf("<pre class=\"full\"><code>prince -s '%s/css/print_%s.css'%s-o /tmp/%s.pdf%s'%s'</code></pre></li>",
				  dirname($selfurl),
				  $orgaform,
				  $rtn,
				  basename(dirname(dirname($selfurl))),
				  $rtn,
				  $selfurl);
    }
    $this->html_body .= "</ul></div>\n";
    $this->html_body .= self::$endhtml;
  }
  
  public function __tostring() {
    if ($this->error_string === '') {
      return $this->html_header.$this->html_body;
    } else {
      return $this->html_header."<h2>Erreur d'analyse</h2>\n".$this->error_string;
    }
  }

}
